// Gouraud-Lighting : calculate color (lighted) for the vertices and interpolate over the pixels

// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tWV: WORLDVIEW;
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)

float4 MAmbient  : COLOR <String uiname="Ambient";>  = {1, 1, 1, 1};
float4 MDiffuse : COLOR <String uiname="Diffuse";>  = {1, 1, 1, 1};
float4 MSpecular : COLOR <String uiname="Specular";> = {1, 1, 1, 1};


//light properties
float3 lDir <string uiname="Light Direction";> = {0, 5, 0}; //Light Position in world space
float4 lAmb  : COLOR <String uiname="Ambient Color";>  = {0, 0, 0, 1};
float4 lDiff : COLOR <String uiname="Diffuse Color";>  = {0.5, 0.5, 0.5, 1};
float4 lSpec : COLOR <String uiname="Specular Color";> = {0.6, 0.6, 0.6, 1};
float lPower <String uiname="Power"; float uimin=0.0;> = 10.0;   //shininess of specular highlight

//texture
texture Tex <string uiname="Texture";>;
float4x4 tTex <string uiname="Texture Transform";>;                  //Texture Transform
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //set the sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 tColor <string uiname="Color Transform";>;


// -------------------------------------------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// -------------------------------------------------------------------------------------------------------------------------------------

//data that ist returned by the vertexshader
struct VS_OUTPUT
{
    float4 PosWVP: POSITION;
    float2 TexCd: TEXCOORD0;
    float4 LightColor: COLOR0;
};

VS_OUTPUT VS(
    float4 PosO  : POSITION,
    float3 NormO : NORMAL,
    float2 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;

    //inverse light direction in view space
    float3 LightDirV = normalize(-mul(lDir, tV));

    //normal in view space
    float3 NormV = normalize(mul(NormO, tWV));

    //view direction = inverse vertexposition in viewspace
    float4 PosV = mul(PosO, tWV);
    float3 ViewDirV = normalize(-PosV);

    //halfvector
    float3 H = normalize(ViewDirV + LightDirV);

    //compute blinn lighting
    float4 shades = lit(dot(NormV, LightDirV), dot(NormV, H), lPower);

    float4 amb = MAmbient * lAmb;
    float4 diff = MDiffuse * lDiff * shades.y;
    diff.a = 1;
    float4 spec = MSpecular * lSpec * shades.z;
    spec.a = 1;

    //position (projected)
    Out.PosWVP  = mul(PosV, tP);
    Out.TexCd = mul(float4(TexCd, 0.0, 1.0 ), tTex);
    Out.LightColor = amb + diff + spec;

    return Out;
}

// -------------------------------------------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------------------------------------------------------------------

float4 PS(
    float2 TexCd : TEXCOORD0,
    float4 LightColor : COLOR0): COLOR
{
    float4 col = tex2D(Samp, TexCd) + LightColor;
    return mul(col, tColor);
}

// -------------------------------------------------------------------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------------------------------------------------------------------

technique TGouraud
{
    pass P0
    {
        VertexShader = compile vs_1_0 VS();
        PixelShader = compile ps_1_4 PS();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transformations
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //material
        MaterialAmbient  = (MAmbient);
        MaterialDiffuse  = (MDiffuse);
        MaterialSpecular = (MSpecular);
        MaterialPower    = (lPower);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;

        //lighting
        LightEnable[0] = TRUE;
        Lighting       = TRUE;
        SpecularEnable = TRUE;

        LightType[0]     = DIRECTIONAL;
        LightAmbient[0]  = (lAmb);
        LightDiffuse[0]  = (lDiff);
        LightSpecular[0] = (lSpec);
        LightDirection[0] = (lDir);

        //shading
        ShadeMode = GOURAUD;
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
